@@ -4,6 +4,7 @@ from django.contrib.contenttypes.models import ContentType
 
 from jsonfield import JSONField
 
+from hellosign import HelloSignUnclaimedDraftDocumentSignature
 from .services import HelloSignService, HelloSignSignerService
 from .models import HelloSignRequest
 
@@ -116,66 +117,89 @@ class HelloSignModelMixin(ModelContentTypeMixin):
         """
         raise OverrideModelMethodException('You must override hs_document and return the pdf/docx/doc file to send to HelloSign')
 
-    def get_hs_service(self):
+    def get_hs_service(self, **kwargs):
         """
         Return the HelloSign service instance with all required to send for
         signing
         """
-        return HelloSignService(document=self.hs_document(),
-                                title=self.hs_document_title(),
-                                invitees=self.hs_signers(),
-                                subject=self.hs_subject(),
-                                message=self.hs_message())
+        kwargs.update({
+            'document': self.hs_document(),
+            'title': self.hs_document_title(),
+            'invitees': self.hs_signers(),
+            'subject': self.hs_subject(),
+            'message': self.hs_message(),
+        })
+        return HelloSignService(**kwargs)
+
+    def create_unclaimed_draft(self, requester_email_address, **kwargs):
+        """
+        Create an unsigned draft that can be used to setup a document for signing
+        """
+        service = self.get_hs_service(HelloSignSignatureClass=HelloSignUnclaimedDraftDocumentSignature)  # override the HelloSignSignatureClass
+
+        resp = service.create_unclaimed_draft(test_mode=HELLOSIGN_TEST_MODE,
+                                              client_id=settings.HELLOSIGN_CLIENT_ID,
+                                              requester_email_address=requester_email_address,
+                                              **kwargs)
+
+        return self.hs_process_result(resp=resp)
 
     def send_for_signing(self, **kwargs):
         """
         Primary method used to send a document for signing
         """
         service = self.get_hs_service()
-        date_sent = str(datetime.datetime.utcnow())
+
         resp = service.send_for_signing(test_mode=HELLOSIGN_TEST_MODE,
                                         client_id=settings.HELLOSIGN_CLIENT_ID,
                                         **kwargs)
-        # resp = service.send_for_signing(test_mode=HELLOSIGN_TEST_MODE)
-        if resp.status_code not in [200, 201, 202]:
-            raise Exception('Could not send document for signing at HelloSign: %s' % resp.json())
 
-        logger.debug('HelloSign Response: %s' % resp.content)
+        return self.hs_process_result(resp=resp)
 
-        result = self.hs_post_process_result(resp=resp)
-        #
-        # Add the date because HelloSign does not provide a date
-        #
-        result.update({
-            'date_sent': date_sent
-        })
+    def hs_process_result(self, resp):
+        if type(resp) not in [dict]:
+            if hasattr(resp, 'json') is False:
+                raise Exception('resp needs to be a dict or a requests response object with .json() method')
+            else:
+                resp = resp.json()  # assume is a requests response object
 
-        hellosign_request = self.hs_record_result(result=result)
+        logger.debug('HelloSign response: %s' % resp)
 
-        #
-        # Update with our set date_sent variable
-        #
-        resp._content = json.dumps(result)
+        # post process the result
+        resp = self.hs_post_process_result(resp=resp)
+
+        self.hs_record_result(result=resp)
 
         return resp
 
     def hs_post_process_result(self, resp):
-        result = resp.json()
+        date_sent = str(datetime.datetime.utcnow())
         #
-        # try return the default signature_request ugly namespace from HS
-        # otherwise jsut return the whole thing
+        # Add the date because HelloSign does not provide a date
         #
-        result = result.get('signature_request', result)
+        resp.update({
+            'date_sent': date_sent
+        })
 
-        return result
+        return resp
 
     def hs_record_result(self, result):
         # setup the hs request object
-        signature_request_id = result.get('signature_request_id') # get id
+        signature_request = result.get('signature_request', {})
+        signature_request_id = signature_request.get('signature_request_id') # get id
 
         if signature_request_id:
+            hs_request_object, is_new = HelloSignRequest.objects.get_or_create(signature_request_id=signature_request_id,
+                                                                               content_object_type=self.get_content_type_object(),
+                                                                               object_id=self.pk)
+            #
+            # Update the request object data dict with the signature_request object
+            #
+            data = hs_request_object.data
+            data.update(result)
+            # save it back
+            hs_request_object.data = data
+            hs_request_object.save(update_fields=['data'])
 
-            return HelloSignRequest.objects.create(signature_request_id=signature_request_id,
-                                                   content_object_type=self.get_content_type_object(),
-                                                   object_id=self.pk,
-                                                   data=result)
+            return hs_request_object
+        return None
