@@ -4,7 +4,8 @@ from django.utils.timezone import utc
 from hellosign import HelloSigner, HelloDoc
 from hellosign import (HelloSignSignature,
                        HelloSignEmbeddedDocumentSignature,
-                       HelloSignEmbeddedDocumentSigningUrl)
+                       HelloSignEmbeddedDocumentSigningUrl,
+                       HelloSignUnclaimedDraftDocumentSignature)
 
 import time
 from datetime import timedelta, datetime
@@ -17,7 +18,7 @@ class AuthenticationSettingsException(Exception):
     message = 'settings.HELLOSIGN_AUTHENTICATION tuple ("username", "password") has not been provided.'
 
 
-class BaseHellSignHelper(object):
+class BaseHelloSignHelper(object):
     hellosign_authentication = None
 
     def __init__(self, **kwargs):
@@ -30,8 +31,14 @@ class BaseHellSignHelper(object):
             logger.critical(msg)
             raise AuthenticationSettingsException(msg)
 
+    def validate(self, resp):
+        if 'error' in resp or resp.status_code not in [200, 201, 202]:
+            raise Exception('HelloSign Api Error: %s' % resp)
 
-class HelloSignService(BaseHellSignHelper):
+        return resp.json()
+
+
+class HelloSignService(BaseHelloSignHelper):
     """
     Service that allows us to send a document for signing
     """
@@ -39,7 +46,7 @@ class HelloSignService(BaseHellSignHelper):
         self.document = document
         logger.info(u'Submitting document to HelloSign: "%s"' % document.name)
 
-        # Dependency injected class for testing
+        # Dependency injected class for testing and using other HS Classes
         self.HelloSignSignatureClass = kwargs.get('HelloSignSignatureClass', HelloSignEmbeddedDocumentSignature)
 
         self.invitees = invitees
@@ -50,22 +57,54 @@ class HelloSignService(BaseHellSignHelper):
 
         super(HelloSignService, self).__init__(**kwargs)
 
-    def send_for_signing(self, **kwargs):
-        signature = self.HelloSignSignatureClass(title=self.title, subject=self.subject, message=self.message)
+    def create_unclaimed_draft(self, **kwargs):
+        unclaimed_draft = HelloSignUnclaimedDraftDocumentSignature(title=self.title, subject=self.subject, message=self.message)
 
         # Add invitees
         for i in self.invitees:
-            signature.add_signer(HelloSigner(name=i['name'], email=i['email']))
+            unclaimed_draft.add_signer(HelloSigner(name=i['name'], email=i['email']))
 
-        signature.add_doc(HelloDoc(file_path=self.document.name))
+        unclaimed_draft.add_doc(HelloDoc(file_path=self.document.name))
 
         # Perform the submission
-        result = signature.create(auth=self.hellosign_authentication, **kwargs)
+        data = {}
 
-        return result
+        resp = self.validate(unclaimed_draft.create(auth=self.hellosign_authentication, **kwargs))
+        data.update(resp)
+
+        # extract the signature_request_id
+        signature_request_id = data.get('unclaimed_draft', {}).get('signature_request_id', None)
+
+        return data
+
+    def send_for_signing(self, **kwargs):
+        signature_request_id = kwargs.pop('signature_request_id', None)
+
+        if signature_request_id is not None:
+            #
+            # Existing signing request use details
+            #
+            signature = self.HelloSignSignatureClass(signature_request_id=signature_request_id)
+            # get the detail
+            return signature.detail(signature_request_id=signature_request_id,
+                                    auth=self.hellosign_authentication)
+        else:
+            #
+            # Is a new signature request add signers and add the doc and 
+            # use create
+            signature = self.HelloSignSignatureClass(title=self.title, subject=self.subject, message=self.message)
+            #
+            # Add invitees
+            for i in self.invitees:
+                signature.add_signer(HelloSigner(name=i['name'], email=i['email']))
+
+            signature.add_doc(HelloDoc(file_path=self.document.name))
+
+            # Perform the submission
+            return self.validate(signature.create(auth=self.hellosign_authentication, **kwargs))
 
 
-class HelloSignSignerService(BaseHellSignHelper):
+class HelloSignSignerService(BaseHelloSignHelper):
     """
     Service that returns various data for and about the signer
     """
@@ -87,7 +126,7 @@ class HelloSignSignerService(BaseHellSignHelper):
 
     def embedded_signature_url(self, signature_id):
         service = HelloSignEmbeddedDocumentSigningUrl(signature_id=signature_id)
-        return service.create(auth=self.hellosign_authentication)
+        return self.validate(service.create(auth=self.hellosign_authentication))
 
     def process(self):
         """
@@ -120,7 +159,7 @@ class HelloSignSignerService(BaseHellSignHelper):
                         logger.debug(u'can get signature url, as current is None or has expired')
 
                         resp = self.embedded_signature_url(signature_id=signer.get('signature_id'))
-                        signing_url_log = self.process_embedded_signature_url(signature=signer, response=resp)
+                        signing_url_log = self.process_embedded_signature_url(signature=signer, resp=resp)
                         #
                         # These are critical 
                         # the sign_url gets used in the search for the users sign_url
@@ -133,32 +172,26 @@ class HelloSignSignerService(BaseHellSignHelper):
                         logger.info('signing_url status_code is invalid has not expired: %s for : %s' % (expires_at, self.signatures[i]))
                         pass
 
-    def process_embedded_signature_url(self, signature, response):
+    def process_embedded_signature_url(self, signature, resp):
         """
         Process the HS response to an embeded signature url
         """
-        if response.status_code not in [200,201]:
-            resp_json = response.json()
-            logger.critical('Could not record the signature url due to: %s, is_new: %s' % (response.status_code, resp_json))
+        resp_json = resp.get('embedded', {})
 
-        else:
-            resp_json = response.json()
-            resp_json = resp_json.get('embedded', {})
+        signature_id = signature.get('signature_id')
 
-            signature_id = signature.get('signature_id')
+        expires_at = datetime.fromtimestamp(resp_json.get('expires_at')).replace(tzinfo=utc)
+        #
+        # Create a SignUrl Object for the record
+        #
+        signing_url_log, is_new = HelloSignSigningUrl.objects.get_or_create(request=self.hellosign_request,
+                                                                            signature_id=signature_id)
+        signing_url_log.expires_at = expires_at
+        signing_url_log.data = resp_json
+        signing_url_log.save(update_fields=['expires_at', 'data'])
 
-            expires_at = datetime.fromtimestamp(resp_json.get('expires_at')).replace(tzinfo=utc)
-            #
-            # Create a SignUrl Object for the record
-            #
-            signing_url_log, is_new = HelloSignSigningUrl.objects.get_or_create(request=self.hellosign_request,
-                                                                                signature_id=signature_id)
-            signing_url_log.expires_at = expires_at
-            signing_url_log.data = resp_json
-            signing_url_log.save(update_fields=['expires_at', 'data'])
-
-            logger.debug('Record the sign_url object: %s, is_new: %s' % (signing_url_log, is_new))
-            return signing_url_log
+        logger.debug('Record the sign_url object: %s, is_new: %s' % (signing_url_log, is_new))
+        return signing_url_log
 
     def sign_url_for_signer(self, email):
         """
